/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <Elementary.h>
#include <app_debug.h>
#include <viewmgr.h>
#include <inputmgr.h>

#include "defs.h"
#include "view.h"
#include "data_photo.h"
#include "utils.h"

#define GENGRID_ITEM_SIZE 200

enum input_handler_type {
	INPUT_HANDLER_TYPE_GRID,
	INPUT_HANDLER_TYPE_NO_CONTENTS,
	INPUT_HANDLER_TYPE_DONE,
	INPUT_HANDLER_TYPE_CANCEL
};

struct _priv {
	Evas_Object *win;
	Evas_Object *base;
	Evas_Object *grid;
	Evas_Object *ly;
	Evas_Object *popup;
	Elm_Gengrid_Item_Class *grid_ic;
	Eina_List *plist;
	Elm_Object_Item *cur;

	struct datamgr *dm;
};

static void _key_down(int id, void *data, Evas *e, Evas_Object *obj,
		Evas_Event_Key_Down *ev);

static void _clicked(int id, void *data, Evas_Object *obj);

static void _mouse_move(int id, void *data, Evas *e, Evas_Object *obj,
		Evas_Event_Mouse_Move *ev)
{
	Elm_Object_Item *it;

	switch (id) {
	case INPUT_HANDLER_TYPE_GRID:
		it = elm_gengrid_at_xy_item_get(obj, ev->cur.canvas.x,
				ev->cur.canvas.y, NULL, NULL);
		if (!it)
			return;

		if (!elm_object_item_focus_get(it))
			elm_object_item_focus_set(it, EINA_TRUE);
		break;
	case INPUT_HANDLER_TYPE_DONE:
	case INPUT_HANDLER_TYPE_CANCEL:
		if (!elm_object_focus_get(obj))
			elm_object_focus_set(obj, EINA_TRUE);
		break;
	default:
		return;
	}
}

static input_handler handler = {
	.key_down = _key_down,
	.clicked = _clicked,
	.mouse_move = _mouse_move
};

static void _delete_popup(struct _priv *priv)
{
	if (!priv->popup)
		return;

	evas_object_del(priv->popup);
	priv->popup = NULL;
}

static void _select_done(struct _priv *priv, Evas_Object *obj)
{
	struct datamgr_item *di;

	_delete_popup(priv);
	elm_object_focus_allow_set(priv->grid, EINA_TRUE);
	di = elm_object_item_data_get(priv->cur);
	if (di && di->icon) {
		viewmgr_update_view(VIEW_USER_EDIT,
				UPDATE_TYPE_PHOTO, di->icon);
				viewmgr_pop_view();
	}
}

static void _key_down(int id, void *data, Evas *e, Evas_Object *obj,
		Evas_Event_Key_Down *ev)
{
	struct _priv *priv = data;

	switch (id) {
	case INPUT_HANDLER_TYPE_GRID:
		/* fallthrough */
	case INPUT_HANDLER_TYPE_NO_CONTENTS:
		if (!strcmp(ev->keyname, KEY_BACK) ||
				!strcmp(ev->keyname, KEY_ESC))
			viewmgr_pop_view();
		break;
	case INPUT_HANDLER_TYPE_DONE:
		if (!data)
			return;

		priv = data;
		if (!strcmp(ev->keyname, KEY_ENTER)) {
			_select_done(priv, obj);
		} else if (!strcmp(ev->keyname, KEY_BACK) ||
				!strcmp(ev->keyname, KEY_ESC)) {
			_delete_popup(priv);
			elm_object_focus_allow_set(priv->grid, EINA_TRUE);
			elm_object_focus_set(priv->grid, EINA_TRUE);
		}
		break;
	case INPUT_HANDLER_TYPE_CANCEL:
		if (!data)
			return;

		priv = data;
		if (!strcmp(ev->keyname, KEY_ENTER) ||
				!strcmp(ev->keyname, KEY_BACK) ||
				!strcmp(ev->keyname, KEY_ESC)) {
			_delete_popup(priv);
			elm_object_focus_allow_set(priv->grid, EINA_TRUE);
			elm_object_focus_set(priv->grid, EINA_TRUE);
		}
		break;
	default:
		return;
	}
}

static void _clicked(int id, void *data, Evas_Object *obj)
{
	struct _priv *priv;

	switch (id) {
	case INPUT_HANDLER_TYPE_DONE:
		if (!data)
			return;

		priv = data;
		_select_done(priv, obj);
		break;
	case INPUT_HANDLER_TYPE_CANCEL:
		if (!data)
			return;

		priv = data;
		_delete_popup(priv);
		elm_object_focus_allow_set(priv->grid, EINA_TRUE);
		elm_object_focus_set(priv->grid, EINA_TRUE);
		break;
	default:
		return;
	}
}

static Evas_Object *_create(Evas_Object *win, void *data)
{
	struct _priv *priv;
	struct datamgr *dm;
	Evas_Object *base;

	if (!win) {
		_ERR("Invalid argument");
		return NULL;
	}

	priv = calloc(1, sizeof(*priv));
	if (!priv) {
		_ERR("failed to calloc priv");
		return NULL;
	}

	dm = datamgr_init(datamgr_photo_get_dclass(), VIEW_PHOTO);
	if (!dm) {
		_ERR("failed to initialize datamgr");
		free(priv);
		return NULL;
	}

	base = utils_add_layout(win, GRP_PHOTO, false, NULL);
	if (!base) {
		_ERR("failed to create base");
		datamgr_fini(dm);
		free(priv);
		return NULL;
	}
	evas_object_size_hint_weight_set(base, EVAS_HINT_EXPAND,
			EVAS_HINT_EXPAND);
	elm_win_resize_object_add(win, base);

	priv->win = win;
	priv->base = base;
	priv->dm = dm;

	viewmgr_set_view_data(VIEW_PHOTO, priv);

	return base;
}

static void _load_no_content(struct _priv *priv)
{
	Evas_Object *ly;

	ly = utils_add_layout(priv->base, GRP_PHOTO_NO_CONTENT, true,
			PART_PHOTO_NO_CONTENT);
	if (!ly) {
		_ERR("failed to create layout");
		return;
	}
	evas_object_show(ly);
	inputmgr_add_callback(ly, INPUT_HANDLER_TYPE_NO_CONTENTS, &handler,
			NULL);

	priv->ly = ly;
}

static Evas_Object *_add_photo_grid(struct _priv *priv)
{
	Evas_Object *grid;

	grid = elm_gengrid_add(priv->base);
	if (!grid) {
		_ERR("failed to add gend grid");
		return NULL;
	}
	evas_object_size_hint_weight_set(grid, EVAS_HINT_EXPAND,
			EVAS_HINT_EXPAND);
	elm_object_part_content_set(priv->base, PART_PHOTO_LIST, grid);
	elm_gengrid_select_mode_set(grid, ELM_OBJECT_SELECT_MODE_ALWAYS);
	elm_gengrid_multi_select_set(grid, EINA_FALSE);
	elm_gengrid_item_size_set(grid,
			ELM_SCALE_SIZE(GENGRID_ITEM_SIZE),
			ELM_SCALE_SIZE(GENGRID_ITEM_SIZE));
	elm_gengrid_horizontal_set(grid, EINA_TRUE);
	elm_scroller_policy_set(grid, ELM_SCROLLER_POLICY_OFF,
			ELM_SCROLLER_POLICY_OFF);
	elm_gengrid_align_set(grid, 0.0, 0.5);
	evas_object_show(grid);

	inputmgr_add_callback(grid, INPUT_HANDLER_TYPE_GRID, &handler, priv);

	return grid;
}

static Evas_Object *_get_grid_content(void *data, Evas_Object *obj,
		const char *part)
{
	struct datamgr_item *di;
	Evas_Object *img = NULL;

	if (!data) {
		_ERR("Invalid argument");
		return NULL;
	}

	di = data;

	if (!strcmp(part, PART_SWALLOW_CONTENT)) {
		img = utils_add_icon(obj, di->icon, NULL);
		if (!img) {
			_ERR("failed to add image");
			return NULL;
		}
	} else if (!strcmp(part, PART_SWALLOW_ICON)) {
		if (di->focus_icon) {
			img = utils_add_icon(obj, di->focus_icon, NULL);
			if (!img) {
				_ERR("failed to add imgae");
				return NULL;
			}
		}
	}

	return img;
}

static void _add_viewer_popup(struct _priv *priv, char *file)
{
	Evas_Object *popup, *ly, *cancel_btn, *done_btn, *img;

	popup = utils_add_popup(priv->base, NULL, NULL);
	if (!popup) {
		_ERR("failed to add popup");
		return;
	}

	ly = utils_add_layout(popup, GRP_PHOTO_VIEWER, false,
			PART_SWALLOW_CONTENT);
	if (!ly) {
		_ERR("failed to add layout");
		evas_object_del(popup);
		return;
	}

	cancel_btn = utils_add_button(ly, MESSAGE_CANCEL,
			PART_PHOTO_VIEWER_CANCEL);
	if (!cancel_btn) {
		_ERR("failed to add cancel button");
		evas_object_del(popup);
		return;
	}
	inputmgr_add_callback(cancel_btn, INPUT_HANDLER_TYPE_CANCEL, &handler,
			priv);

	done_btn = utils_add_button(ly, MESSAGE_DONE, PART_PHOTO_VIEWER_DONE);
	if (!done_btn) {
		_ERR("failed to add done button");
		evas_object_del(popup);
		return;
	}
	inputmgr_add_callback(done_btn, INPUT_HANDLER_TYPE_DONE, &handler,
			priv);

	elm_object_focus_set(cancel_btn, EINA_TRUE);

	img = utils_add_icon(ly, file, PART_PHOTO_VIEWER);
	if (!img) {
		_ERR("failed to add icon");
		evas_object_del(popup);
		return;
	}

	elm_object_focus_set(cancel_btn, EINA_TRUE);
	priv->popup = popup;
}

static void _select_item(void *data, Evas_Object *obj, void *ei)
{
	struct datamgr_item *di;
	struct _priv *priv = data;

	di = elm_object_item_data_get(ei);
	if (!di) {
		_ERR("failed to get data");
		return;
	}

	elm_object_focus_allow_set(priv->grid, EINA_FALSE);
	_add_viewer_popup(priv, di->icon);

	priv->cur = ei;
}

static bool _add_photo_item(struct _priv *priv, Evas_Object *grid,
		Eina_List *dlist)
{
	Elm_Gengrid_Item_Class *ic;
	Elm_Object_Item *item;
	Eina_List *l;
	struct datamgr_item *di;

	elm_gengrid_clear(grid);

	ic = elm_gengrid_item_class_new();
	if (!ic) {
		_ERR("failed to add gengrid item class");
		return false;
	}

	ic->item_style = STYLE_GENGRID_ITEM;
	ic->func.text_get = NULL;
	ic->func.content_get = _get_grid_content;
	ic->func.state_get = NULL;
	ic->func.del = NULL;

	EINA_LIST_FOREACH(dlist, l, di) {
		item = elm_gengrid_item_append(grid, ic, di,
				_select_item, priv);
		if (!item)
			continue;

		elm_object_item_data_set(item, di);
		priv->plist = eina_list_append(priv->plist, item);
	}

	priv->cur = elm_gengrid_first_item_get(grid);
	priv->grid_ic = ic;

	return true;
}

static void _load_photo(struct _priv *priv)
{
	Eina_List *dlist;
	Evas_Object *grid;

	dlist = datamgr_get_items(priv->dm);
	if (!dlist) {
		_load_no_content(priv);
		return;
	}

	grid = _add_photo_grid(priv);
	if (!grid) {
		_ERR("failed to add grid");
		return;
	}

	if (!_add_photo_item(priv, grid, dlist)) {
		_ERR("failed to add photo item");
		evas_object_del(grid);
		_load_no_content(priv);
		return;
	}

	elm_win_focus_highlight_enabled_set(priv->win, EINA_TRUE);
	priv->grid = grid;
}

static void _show(void *data)
{
	struct _priv *priv;

	if (!data) {
		_ERR("Invalid argument");
		return;
	}

	priv = data;

	evas_object_show(priv->base);

	_load_photo(priv);

	if (priv->ly)
		elm_object_focus_set(priv->ly, EINA_TRUE);
	else
		elm_object_focus_set(priv->grid, EINA_TRUE);
}

static void _hide(void *data)
{
	struct _priv *priv;

	if (!data) {
		_ERR("Invalid argument");
		return;
	}

	priv = data;
	if (priv->ly) {
		inputmgr_remove_callback(priv->ly, &handler);
		evas_object_del(priv->ly);
		priv->ly = NULL;
	} else {
		_delete_popup(priv);
		elm_gengrid_item_class_free(priv->grid_ic);
		inputmgr_remove_callback(priv->grid, &handler);
		evas_object_del(priv->grid);
		priv->grid = NULL;
	}

	evas_object_hide(priv->base);
	elm_win_focus_highlight_enabled_set(priv->win, EINA_FALSE);
}

static void _destroy(void *data)
{
	struct _priv *priv;

	if (!data) {
		_ERR("Invalid argument");
		return;
	}

	priv = data;

	datamgr_fini(priv->dm);
	evas_object_del(priv->base);
	free(priv);
}

static view_class vclass = {
	.view_id = VIEW_PHOTO,
	.create = _create,
	.show = _show,
	.hide = _hide,
	.destroy = _destroy
};

view_class *view_photo_get_vclass(void)
{
	return &vclass;
}
