/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <Elementary.h>
#include <app_debug.h>
#include <viewmgr.h>
#include <inputmgr.h>

#include "defs.h"
#include "view.h"
#include "datamgr.h"
#include "utils.h"

#define KEY_ICON "icon"
#define KEY_FOCUS_ICON "focusicon"

#define CTXPOPUP_X 294
#define CTXPOPUP_Y 310
#define MAX_BUF 128
#define MAX_PHOTO 160

#define CHARACTER_PATTERN "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"
#define MAX_LENGTH 32

enum input_handler_type {
	INPUT_HANDLER_TYPE_PHOTO,
	INPUT_HANDLER_TYPE_DELETE,
	INPUT_HANDLER_TYPE_EO,
	INPUT_HANDLER_TYPE_BTN_DONE,
	INPUT_HANDLER_TYPE_BTN_CANCEL,
	INPUT_HANDLER_TYPE_POPUP_BTN_DISCARD,
	INPUT_HANDLER_TYPE_POPUP_BTN_CANCEL,
	INPUT_HANDLER_TYPE_ENTRY
};

enum user_item_type {
	USER_ITEM_NONE = 0x00,
	USER_ITEM_ADD = 0x01,
	USER_ITEM_ADMIN = 0x02,
	USER_ITEM_NORMAL = 0x08
};

struct _priv {
	Evas_Object *win;
	Evas_Object *base;
	Evas_Object *ly;
	Evas_Object *photo;
	Evas_Object *name;
	Evas_Object *icon;
	Evas_Object *focus_icon;
	Evas_Object *done;
	Evas_Object *cancel;
	Evas_Object *ctxpopup;
	Evas_Object *popup;
	Evas_Object *delete;
	Eina_Bool locked;

	Evas_Object *table_item[MAX_ITEM_COUNT];

	char *selected_photo;

	struct datamgr *dm;
	struct datamgr_item *di;

	enum user_item_type user_type;
};

static void _focused(int id, void *data, Evas_Object *obj,
		Elm_Object_Item *item)
{
	switch (id) {
	case INPUT_HANDLER_TYPE_PHOTO:
	case INPUT_HANDLER_TYPE_DELETE:
	case INPUT_HANDLER_TYPE_EO:
		elm_object_signal_emit(obj, SIG_FOCUS, SRC_PROG);
		break;
	case INPUT_HANDLER_TYPE_ENTRY:
		elm_object_signal_emit(obj, SIG_SELECT, SRC_ELM);
		break;
	default:
		 return;
	}
}

static void _unfocused(int id, void *data, Evas_Object *obj,
		Elm_Object_Item *item)
{
	switch (id) {
	case INPUT_HANDLER_TYPE_PHOTO:
	case INPUT_HANDLER_TYPE_EO:
	case INPUT_HANDLER_TYPE_DELETE:
		elm_object_signal_emit(obj, SIG_UNFOCUS, SRC_PROG);
		break;
	case INPUT_HANDLER_TYPE_ENTRY:
		elm_object_signal_emit(obj, SIG_UNSELECT, SRC_ELM);
		break;
	default:
		return;
	}
}

static void _key_down(int id, void *data, Evas *e, Evas_Object *obj,
		Evas_Event_Key_Down *ev);

static void _mouse_down(int id, void *data, Evas *e, Evas_Object *obj,
		Evas_Event_Mouse_Down *ev);

static void _mouse_move(int id, void *data, Evas *e, Evas_Object *obj,
		Evas_Event_Mouse_Move *ev)
{
	if (!elm_object_focus_get(obj))
		elm_object_focus_set(obj, EINA_TRUE);
}

static input_handler handler = {
	.focused = _focused,
	.unfocused = _unfocused,
	.key_down = _key_down,
	.mouse_down = _mouse_down,
	.mouse_move = _mouse_move
};

static void _get_profile(struct _priv *priv, const char **name,
		const char **photo)
{
	*name = elm_entry_entry_get(priv->name);
	elm_image_file_get(priv->icon, photo, NULL);
	*photo = utils_get_icon_from_photo(*photo);
}

static void _delete_popup(struct _priv *priv)
{
	if (!priv->popup)
		return;

	evas_object_del(priv->popup);
	priv->popup = NULL;
}

static void _add_discard_popup(struct _priv *priv)
{
	Evas_Object *popup, *discard_btn, *cancel_btn;

	popup = utils_add_popup(priv->base, MESSAGE_DISCARD,
			MESSAGE_DISCARD_CONTENTS);
	if (!popup) {
		_ERR("failed to add popup");
		return;
	}
	priv->popup = popup;

	discard_btn = utils_add_button(popup, MESSAGE_DISCARD,
			PART_POPUP_BUTTON_1);
	if (!discard_btn) {
		_ERR("failed to add delete btn");
		evas_object_del(popup);
		return;
	}
	inputmgr_add_callback(discard_btn, INPUT_HANDLER_TYPE_POPUP_BTN_DISCARD,
			&handler, priv);

	cancel_btn = utils_add_button(popup, MESSAGE_CANCEL,
			PART_POPUP_BUTTON_2);
	if (!cancel_btn) {
		_ERR("failed to add cancel btn");
		evas_object_del(popup);
		return;
	}
	inputmgr_add_callback(cancel_btn, INPUT_HANDLER_TYPE_POPUP_BTN_CANCEL,
			&handler, priv);

	elm_object_focus_set(cancel_btn, EINA_TRUE);
}

static bool _check_changed_input(struct _priv *priv)
{
	const char *photo, *name;

	/**
	 * Default value
	 *
	 * Icon is user icon or default icon
	 */

	_get_profile(priv, &name, &photo);

	if (priv->di) {
		if (strcmp(photo, priv->di->icon))
			return true;
	} else {
		if (strcmp(name, "") || strcmp(photo, IMAGE_USER_DEFAULT))
			return true;
	}

	return false;
}

static bool _add_btns(struct _priv *priv)
{
	Evas_Object *done, *cancel;

	done = utils_add_button(priv->base, MESSAGE_DONE,
			PART_USER_EDIT_BTN_DONE);
	if (!done) {
		_ERR("failed to add done btn");
		return false;
	}
	inputmgr_add_callback(done, INPUT_HANDLER_TYPE_BTN_DONE, &handler,
			priv);

	cancel = utils_add_button(priv->base, MESSAGE_CANCEL,
			PART_USER_EDIT_BTN_CANCEL);
	if (!cancel) {
		_ERR("failed to add cancel btn");
		evas_object_del(done);
		return false;
	}
	inputmgr_add_callback(cancel, INPUT_HANDLER_TYPE_BTN_CANCEL, &handler,
			priv);

	priv->cancel = cancel;
	priv->done = done;

	return true;
}

static Evas_Object *_add_icon(Evas_Object *photo, const char *file,
		const char *part)
{
	Evas_Object *icon;
	int w, h;

	icon = elm_image_add(photo);
	if (!icon) {
		_ERR("failed to add image");
		return NULL;
	}

	elm_image_file_set(icon, file, NULL);
	elm_image_object_size_get(icon, &w, &h);

	if (w > MAX_PHOTO && h > MAX_PHOTO)
		elm_image_fill_outside_set(icon, EINA_TRUE);
	else
		elm_image_resizable_set(icon, EINA_FALSE, EINA_FALSE);

	elm_object_part_content_set(photo, part, icon);

	return icon;
}

static void _load_user_icon(struct _priv *priv, const char *file,
		const char *focus_file)
{
	Evas_Object *icon, *focus_icon;

	icon = _add_icon(priv->photo, file, PART_USER_EDIT_PHOTO);
	if (!icon)
		_ERR("failed to add icon");

	focus_icon = _add_icon(priv->photo, focus_file,
			PART_USER_EDIT_PHOTO_FOCUS);
	if (!focus_icon)
		_ERR("failed to add focus icon");

	priv->icon = icon;
	priv->focus_icon = focus_icon;
}
static Evas_Object *_pack_icon(Evas_Object *table, struct icon_info *_icon_info,
		int i)
{
	Evas_Object *eo, *ic, *focus_ic;

	eo = utils_add_layout(table, GRP_USER_EDIT_ICON_LIST_ITEM, true, NULL);
	if (!eo) {
		_ERR("failed to add layout");
		return NULL;
	}
	evas_object_show(eo);
	elm_table_pack(table, eo, i % 4, i / 4, 1, 1);

	ic = utils_add_icon(eo, _icon_info->icon_file,
			PART_USER_EDIT_ICON_LIST_ITEM);
	if (!ic) {
		_ERR("failed to add icon");
		evas_object_del(eo);
		return NULL;
	}

	focus_ic = utils_add_icon(eo, _icon_info->focus_icon_file,
			PART_USER_EDIT_ICON_LIST_ITEM_FOCUS);
	if (!focus_ic) {
		_ERR("failed to add focus icon");
		evas_object_del(eo);
		return NULL;
	}
	evas_object_data_set(eo, KEY_ICON, _icon_info->photo_file);

	return eo;
}

static void _dismissed(void *data, Evas_Object *obj, void *ei)
{
	struct _priv *priv = data;

	elm_object_focus_allow_set(priv->photo, EINA_TRUE);
}

static void _set_focus_icon_list(struct _priv *priv)
{
	int i;

	for (i = 0; i < MAX_ITEM_COUNT; i++) {
		if (i/4 == 0) {
			if (i == 0)
				elm_object_focus_next_object_set(priv->table_item[i], priv->table_item[i+1], ELM_FOCUS_RIGHT);
			else if (i == 3)
				elm_object_focus_next_object_set(priv->table_item[i], priv->table_item[i-1], ELM_FOCUS_LEFT);
			else {
				elm_object_focus_next_object_set(priv->table_item[i], priv->table_item[i+1], ELM_FOCUS_RIGHT);
				elm_object_focus_next_object_set(priv->table_item[i], priv->table_item[i-1], ELM_FOCUS_LEFT);
			}
			elm_object_focus_next_object_set(priv->table_item[i], priv->table_item[i+4], ELM_FOCUS_DOWN);
		} else {
			if (i == 4)
				elm_object_focus_next_object_set(priv->table_item[i], priv->table_item[i+1], ELM_FOCUS_RIGHT);
			else if (i == 7)
				elm_object_focus_next_object_set(priv->table_item[i], priv->table_item[i-1], ELM_FOCUS_LEFT);
			else {
				elm_object_focus_next_object_set(priv->table_item[i], priv->table_item[i+1], ELM_FOCUS_RIGHT);
				elm_object_focus_next_object_set(priv->table_item[i], priv->table_item[i-1], ELM_FOCUS_LEFT);
			}
			elm_object_focus_next_object_set(priv->table_item[i], priv->table_item[i-4], ELM_FOCUS_UP);
		}
	}
}

static void _add_icon_list(struct _priv *priv)
{
	Evas_Coord x, y, w, h;
	Evas_Object *ctxpopup, *ly, *table, *eo;
	int i;
	struct icon_info *_icon_info;

	ctxpopup = elm_ctxpopup_add(priv->base);
	if (!ctxpopup) {
		_ERR("failed to add ctxpopup");
		return;
	}
	elm_ctxpopup_horizontal_set(ctxpopup, EINA_TRUE);
	elm_ctxpopup_auto_hide_disabled_set(ctxpopup, EINA_TRUE);
	evas_object_resize(ctxpopup, 812,348);
	evas_object_smart_callback_add(ctxpopup, "dismissed", _dismissed, priv);

	ly = utils_add_layout(ctxpopup, GRP_USER_EDIT_ICON_LIST, false, NULL);
	if (!ly) {
		_ERR("failed to layout");
		evas_object_del(ctxpopup);
		return;
	}
	elm_object_content_set(ctxpopup, ly);
	evas_object_show(ly);
	evas_object_show(ctxpopup);

	evas_object_geometry_get(priv->photo, &x, &y, &w, &h);
	evas_object_move(ctxpopup, ELM_SCALE_SIZE(x + w), ELM_SCALE_SIZE(y-30));
	table = utils_add_table(ly, true, PART_USER_EDIT_ICON_LIST);
	if (!table) {
		_ERR("failed to add table");
		evas_object_del(ctxpopup);
		return;
	}
	elm_table_padding_set(table, ELM_SCALE_SIZE(1), ELM_SCALE_SIZE(1));
	evas_object_size_hint_weight_set(table, EVAS_HINT_EXPAND,
			EVAS_HINT_EXPAND);

	_icon_info = utils_get_icon_info();

	for (i = 0; i < MAX_ITEM_COUNT; i++) {
		eo = _pack_icon(table, &_icon_info[i], i);
		if (!eo)
			continue;

		priv->table_item[i] = eo;
		inputmgr_add_callback(eo, INPUT_HANDLER_TYPE_EO, &handler,
				priv);
		if (i == 0)
			elm_object_focus_set(eo, EINA_TRUE);
	}

	_set_focus_icon_list(priv);

	elm_object_focus_allow_set(priv->name, EINA_FALSE);
	elm_object_focus_allow_set(priv->delete, EINA_FALSE);
	elm_object_focus_allow_set(priv->done, EINA_FALSE);
	elm_object_focus_allow_set(priv->cancel, EINA_FALSE);
	elm_object_focus_allow_set(priv->photo, EINA_FALSE);
	priv->ctxpopup = ctxpopup;
}

static void _entry_text_changed_cb(void *data, Evas_Object *obj, void *event_info)
{
	struct _priv *priv = data;

	const char *text = elm_entry_entry_get(obj);
	if (!text)
		return;

	char temp_buffer[1024] = {0, };
	int length = strlen(text);
	char temp[2] = {0, };

	if (length > 0) {
		elm_object_disabled_set(priv->done, EINA_FALSE);
		strncpy(temp, text, 1);
		temp[1] = '\0';
	} else {
		elm_object_disabled_set(priv->done, EINA_TRUE);
		_DBG("length is 0");
		return;
	}

	if (temp[0] >= '0' && temp[0] <= '9') {
		if (length == 1)
			elm_entry_entry_set(obj, "");
		else if (length > 1) {
			strncpy(temp_buffer, text+1 , length - 1);
			temp_buffer[length-1] = '\0';
			elm_entry_entry_set(obj, temp_buffer);
		}
	}
}

static Evas_Object *_add_layout(struct _priv *priv)
{
	Evas_Object *ly, *photo, *name;
	Elm_Entry_Filter_Accept_Set name_char = {
		.accepted = CHARACTER_PATTERN
	};
	Elm_Entry_Filter_Limit_Size limit_size = {
		.max_char_count = MAX_LENGTH
	};

	ly = utils_add_layout(priv->base, GRP_USER_EDIT_CONTENTS, false,
			PART_USER_EDIT_CONTENTS);
	if (!ly) {
		_ERR("failed to add layout");
		return NULL;
	}

	photo = utils_add_layout(ly, GRP_USER_EDIT_PHOTO, true,
			PART_USER_EDIT_CONTENTS_PHOTO);
	if (!photo)
		goto err;

	name = utils_add_entry(ly, false, true, PART_USER_EDIT_CONTENTS_NAME,
			STYLE_INPUT_NAME);
	if (!name)
		goto err;
	elm_entry_markup_filter_append(name, elm_entry_filter_accept_set,
			&name_char);
	elm_entry_markup_filter_append(name, elm_entry_filter_limit_size,
			&limit_size);

	inputmgr_add_callback(photo, INPUT_HANDLER_TYPE_PHOTO, &handler, priv);
	inputmgr_add_callback(name, INPUT_HANDLER_TYPE_ENTRY, &handler, priv);

	priv->photo = photo;
	priv->name = name;

	evas_object_smart_callback_add(priv->name, "changed", _entry_text_changed_cb, priv);

	return ly;
err:
	_ERR("failed to add layout");
	evas_object_del(ly);
	return NULL;
}

static bool _add_user_edit(struct _priv *priv)
{
	Evas_Object *ly;

	ly = _add_layout(priv);
	if (!ly) {
		_ERR("failed to add edit layout");
		return false;
	}

	if (!_add_btns(priv)) {
		_ERR("failed to add buttons");
		evas_object_del(ly);
		return false;
	}

	priv->ly = ly;

	return true;
}

static Evas_Object *_create(Evas_Object *win, void *data)
{
	struct _priv *priv;
	Evas_Object *base;

	if (!win || !data) {
		_ERR("Invalid argument");
		return NULL;
	}

	priv = calloc(1, sizeof(*priv));
	if (!priv) {
		_ERR("failed to calloc priv");
		return NULL;
	}

	base = utils_add_layout(win, GRP_USER_EDIT, false, NULL);
	if (!base) {
		_ERR("failed to create base");
		free(priv);
		return NULL;
	}
	evas_object_size_hint_weight_set(base, EVAS_HINT_EXPAND,
			EVAS_HINT_EXPAND);
	elm_win_resize_object_add(win, base);

	priv->win = win;
	priv->base = base;
	priv->dm = data;

	if (!_add_user_edit(priv)) {
		_ERR("failed to add user edit layout");
		evas_object_del(base);
		free(priv);
		return NULL;
	}

	viewmgr_set_view_data(VIEW_USER_EDIT, priv);
	viewmgr_add_view(view_photo_get_vclass(), NULL);
	viewmgr_add_view(view_action_menu_get_vclass(), data);

	return base;
}

static void _delete_ctxpopup(struct _priv *priv)
{
	if (!priv->ctxpopup)
		return;

	evas_object_del(priv->ctxpopup);
	priv->ctxpopup = NULL;

	elm_object_focus_allow_set(priv->name, EINA_TRUE);
	elm_object_focus_allow_set(priv->photo, EINA_TRUE);
	elm_object_focus_allow_set(priv->done, EINA_TRUE);
	elm_object_focus_allow_set(priv->cancel, EINA_TRUE);
	elm_object_focus_allow_set(priv->delete, EINA_TRUE);
}

static void _select_eo(struct _priv *priv, Evas_Object *obj)
{
	const char *file, *focus_file;

	_delete_ctxpopup(priv);

	file = evas_object_data_get(obj, KEY_ICON);
	if (!file) {
		_ERR("failed to get file");
		return;
	}

	focus_file = utils_get_focus_photo_from_photo(file);

	if (!strcmp(file, ICON_ADD) || !strcmp(focus_file, ICON_ADD)) {
		viewmgr_push_view(VIEW_PHOTO);
		elm_object_focus_allow_set(obj, EINA_FALSE);
	} else
		_load_user_icon(priv, file, focus_file);
}

static void _reset(struct _priv *priv);

static void _update_user(struct _priv *priv)
{
	const char *name, *photo;
	char buf[MAX_BUF];
	int result;

	_get_profile(priv, &name, &photo);

	if (!strcmp(name, "")) {
		elm_object_focus_set(priv->name, EINA_TRUE);
		return;
	}

	if (priv->di) {
		priv->di->action = ITEM_SELECT_ACTION_UPDATE;
		priv->di->title = strdup(name ? name : "");
		priv->di->icon = strdup(photo ? photo :
				IMAGE_USER_DEFAULT);
		priv->di->parameter = strdup("");
		snprintf(buf, sizeof(buf), "%s %s", name, MESSAGE_USER_UPDATE);
		if (datamgr_select_item(priv->dm, priv->di) !=
				ITEM_SELECT_ERROR_NONE) {
			_reset(priv);
			return;
		}
	} else {
		result = datamgr_add_item(priv->dm, name, photo, "");
		if (result == USER_ADD_ERROR_INVALID_OPERATION) {
			_reset(priv);
			return;
		} else if (result == USER_ADD_ERROR_EXIST_ALREADY)
			snprintf(buf, sizeof(buf), "%s %s", name, MESSAGE_USER_EXIST_ALREADY);
		else
			snprintf(buf, sizeof(buf), "%s %s", name, MESSAGE_USER_ADD);
	}

	utils_add_toast(priv->win, buf);
	priv->di = NULL;
	viewmgr_pop_view();
}

static void _delete_user(struct _priv *priv)
{
	char buf[MAX_BUF];

	priv->di->action = ITEM_SELECT_ACTION_DELETE;
	if (datamgr_select_item(priv->dm, priv->di) != ITEM_SELECT_ERROR_NONE) {
		_ERR("failed to delete user");
		return;
	}

	snprintf(buf, sizeof(buf), "%s %s", priv->di->title,
			MESSAGE_USER_DELETE);
	utils_add_toast(priv->win, buf);
	priv->di = NULL;
	viewmgr_pop_view();
}

static void _select(struct _priv *priv, int id, Evas_Object *obj)
{
	switch (id) {
	case INPUT_HANDLER_TYPE_DELETE:
		_delete_user(priv);
		break;
	case INPUT_HANDLER_TYPE_PHOTO:
		_add_icon_list(priv);
		break;
	case INPUT_HANDLER_TYPE_EO:
		_select_eo(priv, obj);
		break;
	case INPUT_HANDLER_TYPE_BTN_DONE:
		_update_user(priv);
		break;
	case INPUT_HANDLER_TYPE_BTN_CANCEL:
		if (_check_changed_input(priv)) {
			_add_discard_popup(priv);
			return;
		}
		viewmgr_pop_view();
		break;
	case INPUT_HANDLER_TYPE_POPUP_BTN_DISCARD:
		viewmgr_pop_view();
		_delete_popup(priv);
		break;
	case INPUT_HANDLER_TYPE_POPUP_BTN_CANCEL:
		_delete_popup(priv);
		break;
	default:
		return;
	}
}

/*
	FIXME: block action menu
*/
/*
static void _show_action_menu(struct _priv *priv)
{
	struct view_update_data vdata;

	vdata.state = _check_changed_input(priv);
	vdata.type = MENU_TYPE_ADD;

	if (priv->di) {
		if (priv->user_type == USER_ITEM_ADMIN &&
				priv->di->type != USER_ITEM_ADMIN) {
			vdata.type = MENU_TYPE_EDIT;
			vdata.di = priv->di;
		}
	}

	viewmgr_update_view(VIEW_ACTION_MENU, UPDATE_TYPE_MENU, &vdata);
	viewmgr_show_view(VIEW_ACTION_MENU);
}
*/

static void _key_down(int id, void *data, Evas *e, Evas_Object *obj,
		Evas_Event_Key_Down *ev)
{
	struct _priv *priv;

	if (!data) {
		_ERR("Invalid argument");
		return;
	}

	priv = data;
	if (!strcmp(ev->keyname, KEY_BACK) ||
			!strcmp(ev->keyname, KEY_ESC)) {
		switch (id) {
		case INPUT_HANDLER_TYPE_PHOTO:
		case INPUT_HANDLER_TYPE_DELETE:
		case INPUT_HANDLER_TYPE_BTN_CANCEL:
		case INPUT_HANDLER_TYPE_BTN_DONE:
		case INPUT_HANDLER_TYPE_ENTRY:
			if (_check_changed_input(priv)) {
				_add_discard_popup(priv);
				return;
			}
			viewmgr_pop_view();
			break;
		case INPUT_HANDLER_TYPE_EO:
			_delete_ctxpopup(priv);
			break;
		case INPUT_HANDLER_TYPE_POPUP_BTN_DISCARD:
		case INPUT_HANDLER_TYPE_POPUP_BTN_CANCEL:
			_delete_popup(priv);
			break;
		default:
			return;
		}
	} else if (!strcmp(ev->keyname, KEY_ENTER)) {
		_select(priv, id, obj);
		ev->event_flags |= EVAS_EVENT_FLAG_ON_HOLD;
	}
}

static void _mouse_down(int id, void *data, Evas *e, Evas_Object *obj,
		Evas_Event_Mouse_Down *ev)
{
	struct _priv *priv;

	if (!data) {
		_ERR("Invalid argument");
		return;
	}

	priv = data;
	_select(priv, id, obj);
	ev->event_flags |= EVAS_EVENT_FLAG_ON_HOLD;
}

static void _change_entry(Evas_Object *name, Eina_Bool disable,
		const char *text, const char *signal)
{
	elm_entry_entry_set(name, text);
	elm_object_focus_allow_set(name, disable);
	elm_entry_editable_set(name, disable);
	elm_object_signal_emit(name, signal, SRC_ELM);
}

static void _reset(struct _priv *priv)
{
	const char *photo, *focus_photo;

	if (priv->selected_photo) {
		_load_user_icon(priv, priv->selected_photo,
				priv->selected_photo);
		return;
	}

	if (priv->di) {
		photo = utils_get_photo_from_icon(priv->di->icon);
		focus_photo = utils_get_focus_photo_from_photo(photo);
		_load_user_icon(priv, photo, focus_photo);
	} else {
		_change_entry(priv->name, EINA_TRUE, "", SIG_ENABLE);
		_load_user_icon(priv, IMAGE_USER_CURRENT_DEFAULT,
					IMAGE_USER_CURRENT_DEFAULT_FOCUS);
	}
}

static void _add_delete_user(struct _priv *priv)
{
	if (priv->user_type != USER_ITEM_ADMIN ||
			priv->di->type == USER_ITEM_ADMIN) {
		priv->delete = utils_add_layout(priv->ly, GRP_USER_EDIT_DELETE,
				false, PART_USER_EDIT_CONTENTS_DELETE);
		elm_object_signal_emit(priv->delete, SIG_DISABLE, SRC_PROG);
	} else {
		priv->delete = utils_add_layout(priv->ly, GRP_USER_EDIT_DELETE,
				true, PART_USER_EDIT_CONTENTS_DELETE);
		inputmgr_add_callback(priv->delete, INPUT_HANDLER_TYPE_DELETE,
				&handler, priv);
	}
}

static void _show(void *data)
{
	struct _priv *priv;

	if (!data) {
		_ERR("Invalid argument");
		return;
	}

	priv = data;

	if (priv->di) {
		elm_object_part_text_set(priv->base, PART_USER_EDIT_TITLE,
				MESSAGE_EDIT_USER);
		_change_entry(priv->name, EINA_FALSE, priv->di->title,
				SIG_DISABLE);
		_add_delete_user(priv);
	} else {
		elm_object_part_text_set(priv->base, PART_USER_EDIT_TITLE,
				MESSAGE_ADD_USER);
		elm_object_disabled_set(priv->done, EINA_TRUE);
	}

	_reset(priv);

	evas_object_show(priv->base);
	elm_object_focus_set(priv->cancel, EINA_TRUE);
	priv->selected_photo = NULL;
}

static void _hide(void *data)
{
	struct _priv *priv;

	if (!data) {
		_ERR("Invalid argument");
		return;
	}

	priv = data;

	_delete_popup(priv);
	_delete_ctxpopup(priv);

	if (priv->delete) {
		evas_object_del(priv->delete);
		inputmgr_remove_callback(priv->delete, &handler);
		priv->delete = NULL;
	}

	priv->selected_photo = NULL;
	evas_object_hide(priv->base);
}

static void _destroy(void *data)
{
	struct _priv *priv;

	if (!data) {
		_ERR("Invalid argument");
		return;
	}

	priv = data;

	inputmgr_remove_callback(priv->photo, &handler);
	inputmgr_remove_callback(priv->done, &handler);
	inputmgr_remove_callback(priv->cancel, &handler);
	inputmgr_remove_callback(priv->name, &handler);
	viewmgr_remove_view(VIEW_PHOTO);
	viewmgr_remove_view(VIEW_ACTION_MENU);
	evas_object_del(priv->base);
	free(priv);
}

static void _update(void *view_data, int update_type, void *data)
{
	struct _priv *priv;

	if (!view_data) {
		_ERR("Invalid argument");
		return;
	}

	priv = view_data;

	switch (update_type) {
	case UPDATE_TYPE_EDIT:
		priv->di = data;
		break;
	case UPDATE_TYPE_PHOTO:
		priv->selected_photo = data;
		break;
	case UPDATE_TYPE_USER:
		priv->user_type = (enum user_item_type) data;
		break;
	case UPDATE_TYPE_RESET:
		_reset(priv);
		break;
	default:
		_ERR("Invalid type");
		return;
	}
}

static view_class vclass = {
	.view_id = VIEW_USER_EDIT,
	.create = _create,
	.show = _show,
	.hide = _hide,
	.destroy = _destroy,
	.update = _update
};

view_class *view_user_edit_get_vclass(void)
{
	return &vclass;
}
